#!/usr/bin/env python
# $Id: basic_ftpd.py 569 2009-04-04 00:17:43Z billiejoex $

"""A basic FTP server which uses a DummyAuthorizer for managing 'virtual
users', setting a limit for incoming connections.
"""

import os
import sys
import syslog
import threading
import thread
from time import sleep

from pyftpdlib import ftpserver

def log(s):
    print(s)
    syslog.syslog(s)


def on_file_sent(ftp_handler, file):
    log("sent %s" % file)
    t = threading.Timer(1, thread.interrupt_main)
    t.start()

def timeout():
    log("timeout")
    thread.interrupt_main()


def main(argv):

    if len(argv) < 3:
        log("need an IP address and port")
        return 1

    ip = argv[1]
    port = int(argv[2])

    os.chdir(os.path.dirname(os.path.realpath(__file__)))

    log("listen server on %s:%d" % (ip, port))

    # Instantiate a dummy authorizer for managing 'virtual' users
    authorizer = ftpserver.DummyAuthorizer()

    # Define a new user having full r/w permissions and a read-only
    # anonymous user
    authorizer.add_user('atx', 'atx', os.getcwd(), perm='elradfmw')
    authorizer.add_anonymous(os.getcwd())

    # Instantiate FTP handler class
    ftp_handler = ftpserver.FTPHandler
    ftp_handler.authorizer = authorizer

    # Define a customized banner (string returned when client connects)
    ftp_handler.banner = "pyftpdlib %s based ftpd ready." %ftpserver.__ver__

    # Specify a masquerade address and the range of ports to use for
    # passive connections.  Decomment in case you're behind a NAT.
    #ftp_handler.masquerade_address = '151.25.42.11'
    #ftp_handler.passive_ports = range(60000, 65535)

    ftp_handler.on_file_sent = on_file_sent

    # Instantiate FTP server class and listen to 0.0.0.0:21
    address = (ip, port)
    ftpd = ftpserver.FTPServer(address, ftp_handler)

    # set a limit for connections
    ftpd.max_cons = 256
    ftpd.max_cons_per_ip = 5

    log("starting server %s:%d" % (ip, port))
    t = threading.Timer(60, timeout)
    t.start()
    # start ftp server
    ftpd.serve_forever()
    t.cancel()
    log("server finished %s:%d" % (ip, port))
    return 0

if __name__ == '__main__':
    sys.exit(main(sys.argv))
